@@ -1,5 +1,5 @@
 /*
- * au88x0_game.c,v 1.3 2004/04/13 15:26:28 tiwai Exp
+ * au88x0_game.c,v 1.5 2004/06/29 15:55:12 tiwai Exp
  *
  *  Manuel Jander.
  *
@@ -32,6 +32,7 @@
 
 #include <sound/driver.h>
 #include <linux/time.h>
+#include <linux/delay.h>
 #include <linux/init.h>
 #include <sound/core.h>
 #include "au88x0.h"
@@ -79,7 +80,7 @@ static int vortex_game_open(struct gameport *gameport, int mode)
 		hwwrite(vortex->mmio, VORTEX_CTRL2,
 			hwread(vortex->mmio,
 			       VORTEX_CTRL2) | CTRL2_GAME_ADCMODE);
-		wait_ms(VORTEX_GAME_DWAIT);
+		msleep(VORTEX_GAME_DWAIT);
 		return 0;
 	case GAMEPORT_MODE_RAW:
 		hwwrite(vortex->mmio, VORTEX_CTRL2,
@@ -95,7 +96,7 @@ static int vortex_game_open(struct gameport *gameport, int mode)
 
 static int vortex_gameport_register(vortex_t * vortex)
 {
-	if ((vortex->gameport = snd_kcalloc(sizeof(struct gameport), GFP_KERNEL)) == NULL) {
+	if ((vortex->gameport = kcalloc(1, sizeof(struct gameport), GFP_KERNEL)) == NULL) {
 		return -1;
 	};
 	
