@@ -528,12 +528,9 @@ static inline void class_simple_device_remove(int devnum) { return; }
 #endif
 
 /* msleep */
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 6)
-#define msleep(msecs) \
-	do { \
-		set_current_state(TASK_UNINTERRUPTIBLE); \
-		schedule_timeout(((msecs) * HZ + 999) / 1000);	\
-	} while (0)
+#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 0) && LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 6)) || (LINUX_VERSION_CODE < KERNEL_VERSION(2, 4, 29))
+void snd_compat_msleep(unsigned int msecs);
+#define msleep snd_compat_msleep
 #endif
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 0)
@@ -563,16 +560,37 @@ static inline void class_simple_device_remove(int devnum) { return; }
 #include <linux/moduleparam.h>
 #undef module_param_array
 /* we assumme nump is always NULL so we can use a dummy variable */
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 5)
+#define module_param_array(name, type, nump, perm) \
+	static int boot_devs_##name; \
+	static struct kparam_array __param_arr_##name			\
+	= { ARRAY_SIZE(name), &boot_devs_##name, param_set_##type, param_get_##type,	\
+	    sizeof(name[0]), name };					\
+	module_param_call(name, param_array_set, param_array_get, 	\
+			  &__param_arr_##name, perm)
+#else
 #define module_param_array(name, type, nump, perm) \
 	static int boot_devs_##name; \
 	module_param_array_named(name, name, type, boot_devs_##name, perm)
-#endif
+#endif /* < 2.6.5 */
+#endif /* < 2.6.10 */
 
 /* dump_stack hack */
 #ifndef CONFIG_HAVE_DUMP_STACK
 #define dump_stack()
 #endif
 
+#ifndef CONFIG_HAVE_PCI_DEV_PRESENT
+#include <linux/pci.h>
+#ifndef PCI_DEVICE
+#define PCI_DEVICE(vend,dev) \
+	.vendor = (vend), .device = (dev), \
+	.subvendor = PCI_ANY_ID, .subdevice = PCI_ANY_ID
+#endif
+int snd_pci_dev_present(const struct pci_device_id *ids);
+#define pci_dev_present(x) snd_pci_dev_present(x)
+#endif
+
 /* power management compatibility layer */
 #ifdef CONFIG_PM
 #ifdef PCI_OLD_SUSPEND
